body {
	font-size: 10pt;
	margin-left: 8px;
	margin-right: 8px;
	margin-top: 6px;
	margin-bottom: 6px;
	line-height: 24px;
	font-family: sans-serif;
}
table, td, tr {
	color: #3c3c3c;
	font-size: 10pt;
}
td {
	padding: 6px;
}
a:link {
	text-decoration: none;
	color: #2883e2;
}
a:visited {
	text-decoration: none;
	color: #6856cc;
}
a:hover {
	text-decoration: underline;
	color: #186CC4;
}
pre {
	white-space: normal;
}
a:active {
	text-decoration: underline;
	color: #186CC4;
}
.title-table-cell {
	border-top-left-radius: 3px;
	border-top-right-radius: 3px;
	background-color: #F2DFB1;
}
.title-text {
	font-size: 18pt;
	font-weight: bold;
	padding-bottom: 3px;
	color: #404040;
	text-shadow: 0px 2px 0px rgba(247,247,247,1);
}
.title-secondary-text {
	font-size: 11pt;
	padding-top: 3px;
	color: #474747;
	text-shadow: 0px 2px 0px rgba(247,247,247,1);
}
.subtitle-table-cell {
	background-color: #D6D6D6;
}
.subtitle-text {
	font-size: 11pt;
	color: #000000;
	text-shadow: 0px 1px 0px rgba(247,247,247,1);
}
.subsubtitle-table-cell {
	background-color: #E3E3E3;
}
.subsubtitle-text {
	font-size: 10pt;
	color: #474747;
}
.syntax-text {
	font-family: monospace;
	font-size: 10pt;
	font-weight: bold;
	color: #800000;
}
.note-box {
	background-color: #F2F2F2;
	color: #636363;
	font-size: 9pt;
}
.example-box {
	font-family: monospace;
	font-size: 10pt;
	background-color: #f5f5f5;
	border: 1px solid #d5d5d5;
	color: #800000;
	margin-top: 6px;
	margin-bottom: 6px;
	line-height: 16px;
	padding: 10px;
}
.example-box a:link {
	color: #600030;
}
.example-box a:visited {
	color: #600030;
}
.example-box a:hover {
	color: #FF2B46;
}
.example-box a:active {
	color: #FF2B46;
}
.comment-text {
	color: #578A57;
}
.comment-text a:link {
	color: #578A57;
}
.comment-text a:visited {
	color: #578A57;
}
.comment-text a:hover {
	color: #35BD58;
}
.comment-text a:active {
	color: #35BD58;
}
.example-paren {
	color: #8A7000;
}
.example-bracket {
	color: #9C7914;
}
.example-oper {
	color: #9C7914;
}
.example-variable {
	color: #AB6332;
}
.comment-text .example-variable {
	color: #578A57;
}
.comment-text .example-oper {
	color: #578A57;
}
.comment-text .example-paren {
	color: #578A57;
}
.comment-text .example-bracket {
	color: #578A57;
}
.switch-table {
	width: 100%;
	border-collapse: collapse;
}
.switch-title-table-cell {
	font-family:"Lucida Console", Monaco, "Courier New", monospace;
	font-size: 10pt;
	color: #670000;
	background-color: #EDEDED;
}
/* .switch-body-table-cell {
} */
.footer {
	border-top: 1px solid #a2a2a2;
	color: #a2a2a2;
	margin-top: 8px;
	padding-top: 4px;
	font-style: italic;
	font-size: 8pt;
}
