//=============================================================================
//
//   File : KviKvsTreeNodeParameterReturn.cpp
//   Creation date : Fri 30 Jan 2004 01:31:01 by Szymon Stefanek
//
//   This file is part of the KVIrc IRC client distribution
//   Copyright (C) 2004-2010 Szymon Stefanek <pragma at kvirc dot net>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your option) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
//=============================================================================

#include "KviKvsTreeNodeParameterReturn.h"
#include "KviKvsTreeNodeDataList.h"
#include "KviKvsRunTimeContext.h"
#include "KviKvsVariantList.h"
#include "KviLocale.h"

KviKvsTreeNodeParameterReturn::KviKvsTreeNodeParameterReturn(const QChar * pLocation, KviKvsTreeNodeDataList * pDataList)
    : KviKvsTreeNodeInstruction(pLocation)
{
	m_pDataList = pDataList;
	m_pDataList->setParent(this);
}

KviKvsTreeNodeParameterReturn::~KviKvsTreeNodeParameterReturn()
{
	delete m_pDataList;
}

void KviKvsTreeNodeParameterReturn::contextDescription(QString & szBuffer)
{
	szBuffer = "Parameter Return Evaluation";
}

void KviKvsTreeNodeParameterReturn::dump(const char * prefix)
{
	qDebug("%s ParameterReturn", prefix);
	QString tmp = prefix;
	tmp += "  ";
	m_pDataList->dump(tmp.toUtf8().data());
}

bool KviKvsTreeNodeParameterReturn::execute(KviKvsRunTimeContext * c)
{
	KviKvsVariantList lBuffer;
	if(!m_pDataList->evaluate(c, &lBuffer))
		return false;
	if(lBuffer.count() == 0)
	{
		c->returnValue()->setNothing();
		return true;
	}
	if(lBuffer.count() == 1)
	{
		c->returnValue()->copyFrom(*(lBuffer.first()));
		return true;
	}

	QString all;
	lBuffer.allAsString(all);
	c->returnValue()->setString(all);
	return true;
}
