static const char * g_szAboutText = "" \
"<html>" \
"<head>" \
"<title>Honor and glory</title>" \
"</head>" \
"<body bgcolor=\"#FFFFFF\" text=\"#000000\" link=\"#FF0000\"><center>" \
"<h4><font color=\"#000000\">" \
"This is a partial list of the people that have " \
"contributed in some way to the KVIrc project.<br><br>" \
"Honor and glory to:<br>" \
"</font></h4>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Core developers</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Szymon Stefanek</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Pragma" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"pragma at kvirc dot net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"The \"Creator\" of KVIrc" \
"<br>" \
"Mental work, hand work," \
"<br>" \
"eyes consuming and coffee drinking" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"I'll check... lemme check...\"" \
"<br>" \
"\"I'll fix it tomorrow...\"" \
"<br>" \
"\"KVIrc: the client that can't make coffee\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Alessandro Carbone</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Noldor" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"elfonol at gmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Betatesting, bugfixing, exotic scripting, documentation," \
"<br>" \
"code improvements, win32 packages... To be continued :-)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"&lt;Grifisx&gt;...damned brackets  ...there are always too many...\"" \
"<br>" \
"\"&lt;Noldor&gt;....or too few..\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Elvio Basello</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"HelLViS69" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"hellvis69 at gmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Site (re)creator, Italian translation, scripting," \
"<br>" \
"bughunting, Slackware packages, new ideas..." \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"&lt;HelLViS69&gt; Blessed gdb, in the name of ritchie,\"" \
"<br>" \
"\"kermingham and rms\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Alexey Uzhva</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Alexey" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"wizard at opendoor dot ru" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bugfixing, code improvements, localization improvements" \
"<br>" \
"To be continued :-)" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Fabio Bas</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"CtrlAltCa" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"ctrlaltca at gmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bugfixing, coding, mingw32 port, user support" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Can I haz KVIrc?\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Thomas Kalla</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"|Tom|@IrcNet" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"MacOS X porting" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Voker57</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Voker57" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"voker57 at gmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bugfixing, cmake, kde4, other new stuff" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"I'm not sure how I did it, but it works\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>TheXception</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"TheXception" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"kvirc@thexception.net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bugfixing, german tutorials" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"everything is possible, it only needs its time.\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Kai Wasserbäch</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Curan (actually the correct form would be Cùran)" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"debian@carbon-project.org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"The Debian guy in the team. Introduces bug fixes for" \
"<br>" \
"Debian Bugs and added the German manpage." \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"As Debian's Co-Maintainer of KVIrc I have a Debian-centric\"" \
"<br>" \
"\"view on some things, which is - apart from bugs reported to\"" \
"<br>" \
"\"Debian - the main force driving me to improve KVIrc.\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Matt Ullman</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"staticfox" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"staticfox at staticfox dot net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Development and bug fixing" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Codes with his paws.\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Translators</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>José Melo</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Ze_M" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"mmodem00 at netvisao dot pt" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Portuguese translation, default script patches and Mandriva (MDE) packager." \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Ahinu</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Ahinu" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"ahinu.laman at gmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"French translation, code fixes, betatesting and heavy bughunting" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>_Otaku</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"_Otaku" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"notakou at tiscali dot fr" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"French translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>mren</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"mren" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"kvirc at mrmx dot de" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"German translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Ionic</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Ionic" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"ionic at ionic dot de" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"German translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Fiber^</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Fiber^" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"vfiber at gmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Hungarian translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Erkan Kodalak</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Nakre" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"bravehearterkan at hotmail dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Turkish translation and bug hunting" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Sian Mountbatten</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Poenikatu" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"poenikatu at fastmail dot co dot uk" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Esperanto translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Graphics</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Cle'</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"etherea`" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"badgirl at virg0 dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Artist :)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Time makes no sense ;)\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Andidas</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Andidas" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"http://www.andidas.com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Splash screen Artist :)" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>un1versal</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Design, Spellchecking, bug-hunting and minor development." \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Icon set, themes and graphics for KVIrc v4.3.x\"" \
"<br>" \
"\"Kudos to all the KVIrc translators.\"" \
"<br>" \
"\"Translating gibberish is never easy. :D\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Packagers</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Massimiliano Leone</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"k0sm|k0" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"k0smik0 at logorroici dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Debian packages, official femal users supporter" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"One must make one's self superior to humanity, in power,\"" \
"<br>" \
"\"in loftiness of soul, in contempt. - Friedrich Nietzsche\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Services</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Jan Wagner</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Istari" \
", " \
"spion" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Webmaster, bughunter, snapshot maintainer" \
"<br>" \
"support gay^Wguy :o) and the man that gives" \
"<br>" \
"you KVIrc through IPv6." \
"</font></td></tr>" \
"</table><br><br>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Contributors</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Tomasz Moń</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"desowin" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"desowin at gmail dot com" \
"]" \
"</font></td></tr>" \
"</table><br><br>" \
"<br><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+3\"><b>Historic project staff</b></font>" \
"</td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Luk De Ketelaere</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Balboy" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"balboy at kvirc dot net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Documentation, bug searching, public relations" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"No idea of what you're talking about... or why you need it :)\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Andrea Parrella</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"YaP" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"yap at kvirc dot net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bug hunting, plugin development, bug fixes," \
"<br>" \
"windows porting and snapshots" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"why you haven't implemented a class like DoEverythingByYourself ? :)\"" \
"<br>" \
"\"&lt;Pragma&gt; a wonderful recursive function :)\"" \
"<br>" \
"\"&lt;YaP&gt; Damn...I knew it eheheheheh\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Christian Strommen</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Number1" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"number1 at realityx dot net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Makes sure that nobody is lazy, bug-hunter, a LOT of suggestions and maker of RealityX IRC" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Skydiving is almost as exciting as KVIrc scripting!\"" \
"<br>" \
"\"Pragma: could you add these functions: $rubmyback() and $makecoffe() ?\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Olle Hällnäs</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Crocodile" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"olle at xmms dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"The \"open hand\" for the KVIrc project." \
"<br>" \
"Provider of the CVS service, www & ftp mirroring, bandwidth," \
"<br>" \
"great ideas and fun while coding." \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Juanjo Alvarez</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Kane" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"juanjux at yahoo dot es" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Away system, colored nicks, general programming, spanish translantation" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Pragma: think of KVIrc as a tool for scripters... we have to left them something to do :)\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Krzysztof Godlewski</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Kristoff" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"kristoff at proczta dot wprost dot pl" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Official polish beer provider" \
"<br>" \
"and plugin developer" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"echo \"STOP IT\" &gt; /dev/neighbour\"" \
"<br>" \
"\"cd ./neighbour && make clean\"" \
"<br>" \
"\"Pragma...erm...bugs ? WHICH bugs ? ;))\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Christopher R. Gabriel</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"CGabriel" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"cgabriel at linux dot it" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"The 'light' inside the GNU clouds" \
"<br>" \
"Generic (?!?) official project supporter :)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Pragma: seems that kvirc with all its plugins is becoming a desktop environment\"" \
"<br>" \
"\"echo espresso &gt; /dev/coffee\"" \
"<br>" \
"\"Pragma: you reached the user limit. WTF!\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Andrew Frolov</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Drosha@Undernet" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Beta testing" \
"<br>" \
"Ideator of www.kvirc.org" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"This free project is better that shareware\"" \
"<br>" \
"\"mIRC for windows even in alpha stage\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Ted Podgurski</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"alch3m1st" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"tedp at toocool dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Test scripter, bug hunting" \
"<br>" \
"ideator & creator of the first www.kvirc.net" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"whassup ?\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>RAD Kade 1</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"RADKade1" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Debugging an improvements" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"I'm just *really* good at doing things\"" \
"<br>" \
"\"when I have no idea of what I'm actually doing\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Fritz Elfert</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"KNFritz@OpenProjects" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Improvements of KVIrc code and of my linux knowledge" \
"<br>" \
"I like to think him as a 'spiritual guide'... :D" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Charles Samuels</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Njaard" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"charles at altair dot dhs dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"The KDE man! Making sure that KVIrc has a reason to have a K in the beginning" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Could I please make KVIrc a pure KDE application today?? Pleeease...\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>BuTi</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"BuTi@OpenProjects" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"buti at geocities dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Some \"tiny\" pieces of code" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Nick Betcher</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Error403" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"nbetcher at real-time dot com" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"&lt;Error403&gt; Number1: I imagined you looking crazy :)\"" \
"<br>" \
"\"\"&lt;NumeroUno&gt; I was drunk\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Orn E. Hansen</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"oehansen" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"oe.hansen at gamma dot telenordia dot se" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"The official C++ manual and plugin developer" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"prepare yourselves..\"" \
"<br>" \
"\"#include &lt;iostream.h&gt;\"" \
"<br>" \
"\"class testing {...\"" \
"<br>" \
"\"&lt;oehansen&gt; I must admit, I'm getting confused :-)\"" \
"<br>" \
"\"&lt;Pragma&gt; Me too, at this point :)\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Giovanni Ferri</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"marl" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"giovanni at fonderiadigitale dot it" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bug hunter, ideas source :)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"buy a pigeon traveller and pass dcc transfers to him\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Anodine</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"[Anodine]" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Various funky improvements" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"I can program anything I set my mind to!\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Paul Boehm</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"infected" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Beta testing, language structure improvements" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"...it's enough for 100 IPs per meter\"" \
"<br>" \
"\"on earth, so every lightswitch can have\"" \
"<br>" \
"\"its own IP...then you telnet to your lightswitch\"" \
"<br>" \
"\"and turn it off!\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>munehiro</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"munehiro" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Italian translations, code improvements and many useful suggestions" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Eva Schaller</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Penny" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Italian translation, bug hunting, moral support :))))" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"KVIrc ? Ah! Il dinosauro...\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Christoph Thielecke</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Crissi" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"German translation, bug hunting, new ideas" \
"<br>" \
"Some scripting, Code improvements + new code ;)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Pragma: Short question...\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Triskelios</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Triskelios" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"trisk-kvirc at quasarnet dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Breaking everything, and correcting grammar when there isn't anything left to break" \
"<br>" \
"Also, listening to Aeri's requests and getting about half of them done..." \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Hey, it's not my responsibility to find a funny quote of myself ^_~\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Dušan Hokův</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Hunterz" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"dusan at mirc dot cz" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Czech translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Thilo Shulz</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Thilo" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bugtracking and constant user support on #kvirc" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Iacopo Palazzi</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Iakko@IrcNet" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"iakko at omnikron dot net" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Various code patches, ideas and bug hunting" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Valeriy Girchenko</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Russian translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Adilson Gonçalves Soares Junior</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"CaBiDe" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"cabide at brfree dot com dot br" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Portuguese Brazilian translation" \
"</font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Tonino Imbesi</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"Grifisx" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"grifisx at barmes dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Betatesting, bugfixing, exotic scripting," \
"<br>" \
"code improvements, documentation....To be continued :-)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"&lt;Grifisx&gt; The difference between me and you is just that\"" \
"<br>" \
"\"&lt;Grifisx&gt; You see things and you say \"Why?\" But I dream things\"" \
"<br>" \
"\"&lt;Grifisx&gt; that never were and I say...\"Why not?\"\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Lorenzo Marcon</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"LoSko" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"http://losko.altervista.org/nglfserv/" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"Bug hunting and fixing, doc fixes, scripting (NGL File Server)" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"I don't know what I want, but I know how to get it.\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" \
"<tr><td align=\"center\">" \
"<font color=\"#000000\" size=\"+2\"><b>Max Brazhnikov</b></font>" \
"</td></tr>" \
"<tr><td align=\"center\"><font color=\"#800020\" size=\"+2\"> (" \
"makc" \
")</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#0000FF\" size=\"+1\">" \
"[" \
"makc at FreeBSD dot org" \
"]" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#303030\">" \
"FreeBSD Port maintainer" \
"</font></td></tr>" \
"<tr><td align=\"center\"><font color=\"#606060\"><i>" \
"\"Ports committer\"" \
"</i></font></td></tr>" \
"</table><br><br>" \
"<br>" \
"This list is far from being complete.<br>" \
"A lot of more or less \"anonymous\" people have wasted their time, " \
"eyes, bandwidth and brain resources on this project.<br>" \
"If you're one of those, then well, this dialog is also for you :)<br>" \
"</center><body>" \
"</html>" \
"";
